==> IceWarpServer-9.1.0_RHEL5.tar.gz: html/challenge/inc/smime.php <==
<?

require_once('inc/tools.php');

/**
 * @brief		Class for s/mime message handling
 */
class SMime
{
	public static function processMessage(&$message, $recipient, $sign, $encrypt)
	{
		$plainfile = self::convertMessageToPlainfile($message);
		$headerarray = self::getOpenSSLheaderarray($message);

		// Sign with the sender's private key
		if ($sign)
		{
			$encodedfile = self::randomFilename();
			$cert = self::getUserCertificate();

			if ($cert) {
				$key = $cert['data'];
				$passphrase = $cert['passphrase'];
				if ($passphrase) 
					$fcert = array($key, $passphrase); 
				else 
					$fcert = $key;
	
				if ($key) 
					if (@openssl_pkcs7_sign($plainfile, $encodedfile, $key, $fcert, $headerarray))
					{
						@unlink($plainfile);
						$plainfile = $encodedfile;
					} 
					else 
						@unlink($encodedfile);
			}
		}

		// Encrypt with the recipients public key
		if ($encrypt)
		{
			$encodedfile = self::randomFilename();
			$key = self::getRecipientCertificate($recipient);

			if ($key) 
				if (@openssl_pkcs7_encrypt($plainfile, $encodedfile, $key, $headerarray))
				{
					@unlink($plainfile);
					$plainfile = $encodedfile;
				} 
				else 
					@unlink($encodedfile);
		}

		// Load file back
		$message = file_get_contents($plainfile);
		$message = Tools::cr2crlf($message);
		$message = Tools::dotConvert($message);

		// Delete file
		@unlink($plainfile);
	}

	public static function decodeMessage(&$certmsg)
	{
		// Save To File
		$encodedfile = self::randomFilename();
		$plainfile = self::randomFilename();
		if (@$fd = fopen($encodedfile, "wb"))
		{
			fwrite($fd, $certmsg, strlen($certmsg));
			fclose($fd);
		}

		// Get Private Key
		$cert = self::getUserCertificate();

		$key = $cert['data'];
		$passphrase = $cert['passphrase'];
		if ($passphrase) 
			$fcert = array($key, $passphrase); 
		else 
			$fcert = $key;

		// Decrypt
		if (@openssl_pkcs7_decrypt($encodedfile, $plainfile, $key, $fcert)) {}
		@unlink($encodedfile);
		
		if (file_exists($plainfile)) {
			$result = false;
			if (filesize($plainfile) > 0) {
				// Load file back
				@$certmsg = file_get_contents($plainfile);
				$certmsg = Tools::cr2crlf($certmsg);
				$certmsg = Tools::dotConvert($certmsg);
				$result = true;
			}

			// Delete file
			@unlink($plainfile);
			
			return $result;
		}
		
		return false;
	}
	
	public static function verify(&$message)
  {
    $filename = self::randomFilename();
    if (!$message || !file_put_contents($filename, $message))
    	return false;

    if (openssl_pkcs7_verify($filename,0) === true) 
			$result = true;
		else
			$result = false;
			
    @unlink($filename);

    return $result;
  } 

	public static function getCertificateInfo($key = '')
	{
		$result = '';

		// Get user's certificate
		if (!$key)
		{
			$cert = self::getUserCertificate();
			$key = $cert['data'];
		}

		if ($key)
		{
			 // Get key info
			@$array = openssl_x509_parse($key);

			// Parse cert array
			if (is_array($array))
			{
				$result .= $array['name'] . "\r\n\r\n";

				$result .= $array['subject']['CN'] . "\r\n";
				$result .= $array['subject']['emailAddress'] . "\r\n\r\n";

				$result .= $array['issuer']['C'] . "\r\n";
				$result .= $array['issuer']['ST'] . "\r\n";
				$result .= $array['issuer']['L'] . "\r\n";
				$result .= $array['issuer']['O'] . "\r\n";
				$result .= $array['issuer']['OU'] . "\r\n";
				$result .= $array['issuer']['CN'] . "\r\n";
				$result .= $array['issuer']['emailAddress'] . "\r\n\r\n";

				$result .= $array['validFrom'] . "\r\n";
				$result .= $array['validTo'] . "\r\n\r\n";
			}
		}

		return trim($result);
	}

	private static function getRecipientCertificate($email)
	{
		require_once('inc/gw/gwmanage.php');
		
		$gwMGMT = new GroupWareManagement();
		return $gwMGMT->getEmailCertificate($email);
	}

	private static function getUserCertificate()
	{
		require_once('inc/storage/storage.php');
		
		return Storage::getCertificates();
	}
	
	private static function randomFilename() {
		$filename = eregi_replace("e","z",uniqid('',true));
		return User::getDir() . $filename . '.temp';
	}

	private static function convertMessageToPlainfile($message)
	{
		$filename = self::randomFilename();
		$message = str_replace("\x0D\x0A.", "\x0D\x0A", $message);

		if (@$file = fopen($filename, "wb"))
		{
			fwrite($file, $message, strlen($message));
			fclose($file);
		}
		else
			return false;

		return $filename;
	}

	private static function getOpenSSLheaderarray(&$message)
	{
		$parser = &new MailParse($message);
		$headers = $parser->getHeaders(true);
		$sslHeaders = array("Subject", "From", "Reply-To", "To", "Date", 
												"Return-Receipt", "Message-ID");
		$result = array();
		
		// add all well known headers
		foreach ($sslHeaders as $item) {
			if (isset($headers[strtolower($item)]))
				$result[$item] = $headers[strtolower($item)];
		}
		
		// add all X-* headers
		foreach ($headers as $key => $value) {
			if (substr($key, 0, 2) == 'x-')
				$result['X-'.ucwords(substr($key, 2))] = $value;
		}

		return $result;
	}

}
?>

==> IceWarpServer-9.4.1_RHEL5.tar.gz: html/challenge/inc/smime.php <==
<?

require_once('inc/tools.php');

/**
 * @brief		Class for s/mime message handling
 */
class SMime
{
	public static function processMessage(&$message, $recipient, $sign, $encrypt)
	{
		$plainfile = self::convertMessageToPlainfile($message);
		$headerarray = self::getOpenSSLheaderarray($message);

		// Sign with the sender's private key
		if ($sign)
		{
			$encodedfile = self::randomFilename();
			$cert = self::getUserCertificate();

			if ($cert) {
				$key = $cert['data'];
				$passphrase = $cert['passphrase'];
				if ($passphrase) 
					$fcert = array($key, $passphrase); 
				else 
					$fcert = $key;
	
				if ($key) 
					if (@openssl_pkcs7_sign($plainfile, $encodedfile, $key, $fcert, $headerarray))
					{
						@unlink($plainfile);
						$plainfile = $encodedfile;
					} 
					else 
						@unlink($encodedfile);
			}
		}

		// Encrypt with the recipients public key
		if ($encrypt)
		{
			$encodedfile = self::randomFilename();
			$key = self::getRecipientCertificate($recipient);

			if ($key) 
				if (@openssl_pkcs7_encrypt($plainfile, $encodedfile, $key, $headerarray))
				{
					@unlink($plainfile);
					$plainfile = $encodedfile;
				} 
				else 
					@unlink($encodedfile);
		}

		// Load file back
		$message = file_get_contents($plainfile);
		$message = Tools::cr2crlf($message);
		$message = Tools::dotConvert($message);

		// Delete file
		@unlink($plainfile);
	}

	public static function decodeMessage(&$certmsg)
	{
		// Save To File
		$encodedfile = self::randomFilename();
		$plainfile = self::randomFilename();
		if (@$fd = fopen($encodedfile, "wb"))
		{
			fwrite($fd, $certmsg, strlen($certmsg));
			fclose($fd);
		}

		// Get Private Key
		$cert = self::getUserCertificate();

		$key = $cert['data'];
		$passphrase = $cert['passphrase'];
		if ($passphrase) 
			$fcert = array($key, $passphrase); 
		else 
			$fcert = $key;

		// Decrypt
		if (@openssl_pkcs7_decrypt($encodedfile, $plainfile, $key, $fcert)) {}
		@unlink($encodedfile);
		
		if (file_exists($plainfile)) {
			$result = false;
			if (filesize($plainfile) > 0) {
				// Load file back
				@$certmsg = file_get_contents($plainfile);
				$certmsg = Tools::cr2crlf($certmsg);
				$certmsg = Tools::dotConvert($certmsg);
				$result = true;
			}

			// Delete file
			@unlink($plainfile);
			
			return $result;
		}
		
		return false;
	}
	
	public static function verify(&$message)
  {
    $filename = self::randomFilename();
    if (!$message || !file_put_contents($filename, $message))
    	return false;

    if (openssl_pkcs7_verify($filename,0) === true) 
			$result = true;
		else
			$result = false;
			
    @unlink($filename);

    return $result;
  } 

	public static function getCertificateInfo($key = '')
	{
		$result = '';

		// Get user's certificate
		if (!$key)
		{
			$cert = self::getUserCertificate();
			$key = $cert['data'];
		}

		if ($key)
		{
			 // Get key info
			@$array = openssl_x509_parse($key);

			// Parse cert array
			if (is_array($array))
			{
				$result .= $array['name'] . "\r\n\r\n";

				$result .= $array['subject']['CN'] . "\r\n";
				$result .= $array['subject']['emailAddress'] . "\r\n\r\n";

				$result .= $array['issuer']['C'] . "\r\n";
				$result .= $array['issuer']['ST'] . "\r\n";
				$result .= $array['issuer']['L'] . "\r\n";
				$result .= $array['issuer']['O'] . "\r\n";
				$result .= $array['issuer']['OU'] . "\r\n";
				$result .= $array['issuer']['CN'] . "\r\n";
				$result .= $array['issuer']['emailAddress'] . "\r\n\r\n";

				$result .= $array['validFrom'] . "\r\n";
				$result .= $array['validTo'] . "\r\n\r\n";
			}
		}

		return trim($result);
	}

	private static function getRecipientCertificate($email)
	{
		require_once('inc/gw/gwmanage.php');
		
		$gwMGMT = new GroupWareManagement();
		return $gwMGMT->getEmailCertificate($email);
	}

	private static function getUserCertificate()
	{
		require_once('inc/storage/storage.php');
		
		return Storage::getCertificates();
	}
	
	private static function randomFilename() {
		$filename = eregi_replace("e","z",uniqid('',true));
		return User::getDir() . $filename . '.temp';
	}

	private static function convertMessageToPlainfile($message)
	{
		$filename = self::randomFilename();
		$message = str_replace("\x0D\x0A.", "\x0D\x0A", $message);

		if (@$file = fopen($filename, "wb"))
		{
			fwrite($file, $message, strlen($message));
			fclose($file);
		}
		else
			return false;

		return $filename;
	}

	private static function getOpenSSLheaderarray(&$message)
	{
		$parser = &new MailParse(false,false,false,false,false,$message);
		$headers = $parser->getHeaders(true);
		$sslHeaders = array("Subject", "From", "Reply-To", "To", "Date", 
												"Return-Receipt", "Message-ID");
		$result = array();
		
		// add all well known headers
		foreach ($sslHeaders as $item) {
			if (isset($headers[strtolower($item)]))
				$result[$item] = $headers[strtolower($item)];
		}
		
		// add all X-* headers
		foreach ($headers as $key => $value) {
			if (substr($key, 0, 2) == 'x-')
				$result['X-'.ucwords(substr($key, 2))] = $value;
		}

		return $result;
	}

}
?>
